#!/usr/bin/php
<?php

$title = 'SYNO_ENCODER - Encode your PHP script using syno_compiler';

$help = <<<HELP
$title

Usage: syno_encoder [-f] [-q] -o FILE    file1.php
       syno_encoder [-f] [-q] -o OUTDIR  file1.php file2.php ...
       syno_encoder [-f] [-q] -o OUTDIR  -s SRCDIR  [-e SUFFIX] [-r] [-c] [-l]
       syno_encoder [-f] [-q] -k PRE_KEY -o FILE    file1.php
       syno_encoder [-f] [-q] -k PRE_KEY -o OUTDIR  file1.php file2.php ...
       syno_encoder [-f] [-q] -k PRE_KEY -o OUTDIR  -s SRCDIR  [-e SUFFIX] [-r] [-c] [-l]

  -o FILE   	: the file name to write the encoded script
              	(default to '-encoded.XXX' suffix)
  -o OUTDIR 	: the directory to write all encoded files

  -a SRCDIR
  -s SRCDIR 	: encode all files in this source directory

  -r        	: encode directories recursively (no by default)
  -f        	: force overwriting even if the target exists
  -e SUFFIX 	: encode the files with the SUFFIX extension only (default: php)
            	  (regular expression allowed, ex: "php|inc")
  -c        	: copy files those shouldn't be encoded (no by default)
  -l        	: follow symbolic link (no by default)
  -q        	: do not print the file name while encoding or copying
  -b
  -bz2      	: compress the encoded files with bz2 (needs bzip2-extension)
  -k PRE_KEY	: the pre_key used to generate the key block by syno_compiler

HELP;

if( !function_exists( 'syno_compiler_write_file' ) ) {
        print "$title\n\n";
        print "ERROR: Please install `syno_compiler' before running this encoder\n\n";
        exit;
}

$suffix = 'php|html|htm';
$output = $srcdir = '';
$infiles = array();
$verbose = true;
$compress = false;
$follow_symlink = $force_overwrite = $copy_rest = $recursive = false;
$basename = false;
$pre_key = 0;

for( $i=1 ; $i < $_SERVER['argc']; $i++ ) {
        switch( $_SERVER['argv'][$i] ) {
        case '-h':
                syno_encoder_print_usage();
                break;

        case '-b':
        case '-bz2':
                if( !function_exists('bzopen') )
                        syno_encoder_error('bzip2 extension is not installed');
                $compress = true;
                break;

        case '-f':
                $force_overwrite = true;
                break;

        case '-o':
                if( ++$i < $_SERVER['argc'] )
                        $output = $_SERVER['argv'][$i];
                break;

        case '-s':
        case '-a':
                if( ++$i < $_SERVER['argc'] )
                        $srcdir = $_SERVER['argv'][$i];
                break;

        case '-e':
                if( ++$i < $_SERVER['argc'] )
                        $suffix = $_SERVER['argv'][$i];
                break;

        case '-c':
                $copy_rest = true;
                break;

        case '-r':
                $recursive = true;
                break;

        case '-l':
                $follow_symlink = true;
                break;

        case '-q':
                $verbose = false;
                break;

	case '-k':
		if( ++$i < $_SERVER['argc'] ) {
                        $pre_key = $_SERVER['argv'][$i];
		}
		break;

        default:
                if( preg_match( '/^-o(.+)$/', $_SERVER['argv'][$i], $match ) ) {
                        $output = $match[1];
                }
                else {
                        $infiles[] = $_SERVER['argv'][$i];
                }
        }
}


$numfiles = count($infiles);

$outdir = '';
if( $numfiles > 0 || $srcdir != '' )
        $outdir = $output;

if( $srcdir == '' ) {
        if( $numfiles == 0 )
                syno_encoder_print_usage();
        if( $numfiles > 1 && $outdir == '' )
                syno_encoder_error("You should use `-o DIR' to specify the output directory");
}
else {
        if( $numfiles > 0 )
                syno_encoder_error("You can not encode files and specify `-a DIR' at the same times");
        if( $outdir == '' )
                syno_encoder_error("You should use `-o DIR' to specify the output directory");
}

if( $outdir != '' ) {
        if( file_exists( $outdir ) ) {
/* SYNO: skip folder check */
/*
                if( !is_dir($outdir) )
                        bencoder_error("[$outdir] already exists and is not a directory");
*/
                if( !$force_overwrite )
                        syno_encoder_error("The directory [$outdir] already exists,\n\tuse -f to force overwriting
");
        }
        elseif( $numfiles > 1 )
                mkdir( $output, 0755 );
}


if( $srcdir != '' ) {
        syno_encoder_process_dir( $srcdir, $output );
}
elseif( $numfiles == 1 ) {
        if( $output == '' ) {
                $ext = substr( strrchr( $infiles[0], '.' ), 1 );
                if( $ext )
                        $output = basename( $infiles[0], ".$ext" ) . "-encoded.$ext";
                else
                        $output = $infiles[0] . '-encoded';
        }
        elseif( is_dir($output) )
                $output = $outdir.'/'.$infiles[0];
        if( file_exists($output) && !$force_overwrite )
                syno_encoder_error("The file [$output] already exists,\n\tuse -f to force overwriting");
        if( $verbose )
                print "$title\n\n";
        syno_encoder_encode_file( $infiles[0], $output );
}
else {
        if( $verbose )
                print "$title\n\n";
        foreach( $infiles as $infile ) {
                syno_encoder_encode_file( $infile, "$outdir/$infile" );
        }
}

exit(0);

function syno_encoder_process_dir( $srcdir, $outdir ) {
        global $follow_symlink, $copy_rest, $suffix;
        global $recursive, $force_overwrite;
        $dh = opendir($srcdir);
        while (($file = readdir($dh)) !== false) {
                if( $file == '.' || $file == '..' )
                        continue;
                $srcpath = "$srcdir/$file";
                $outpath = "$outdir/$file";
                if( is_link($srcpath) ) {
                        $real = readlink($srcpath);
                        $realpath = ($real[0]=='/')?$real:"$srcdir/$real";
                        if( $follow_symlink ) {
                                $srcpath = $realpath;
                        }
                        else {
                                if( $recursive || (!is_dir($realpath) && $copy_rest) ) {
                                        if( file_exists($outpath) && $force_overwrite )
                                                @unlink($outpath);
                                        symlink( $real, $outpath );
                                        syno_encoder_show_verbose("symlink: $outpath");
                                }
                                else
                                        syno_encoder_show_verbose("skipped: $outpath");
                                continue;
                        }
                }
                if( is_dir($srcpath) ) {
                        if( $recursive ) {
                                if( !file_exists( $outpath ) ) {
                                        syno_encoder_show_verbose("  mkdir: $outpath");
                                        mkdir($outpath,0755);
                                }
                                elseif( !is_dir($outpath) )
                                        syno_encoder_error("$outpath is not a directory");
                                syno_encoder_process_dir( $srcpath, $outpath );
                        }
                }
                elseif( is_readable($srcpath) ) {
                        if( file_exists($outpath) && !$force_overwrite )
                                syno_encoder_show_verbose("skipped: $outpath");
                        elseif( preg_match( "/\.($suffix)\$/", $file ) ) {
                               	syno_encoder_encode_file( $srcpath, $outpath );
                        } elseif( $copy_rest ) {
                                if( @copy( $srcpath, $outpath ) )
                                        syno_ncoder_show_verbose(" copied: $outpath");
                                else
                                        print "   copy: $outpath (failed)\n";
                        }
                        continue;
                }
                else
                        print " failed: $outpath (un-readable)\n";
        }
        closedir($dh);
}

function syno_encoder_encode_file( $infile, $outfile ) {
	global $basename, $compress, $key_file, $pre_key;
        $fh = fopen( $outfile, 'w');

	$infile_striped = php_strip_whitespace($infile);

	if (0 == $pre_key) {
		$pre_key = rand(1, 100000);
	}

	syno_compiler_write_header($fh, $pre_key);
	syno_compiler_write_file($fh, $infile_striped, $infile);
        syno_compiler_write_footer($fh);
        fclose($fh);
        if( $compress ) {
                $content = file_get_contents($outfile);
                $bzfh = bzopen($outfile, "w");
                bzwrite($bzfh, $content);
                bzclose($bzfh);
                syno_encoder_show_verbose("encoded & compressed: $outfile");
	} else {
                syno_encoder_show_verbose("encoded: $outfile");
	}
        return(1);
}

function syno_encoder_error( $err ) {
        global $title;
        print "$title\n\n";
        print "ERROR: $err\n\n";
        exit(1);
}

function syno_encoder_print_usage() {
        global $help;
        print $help;
        exit(0);
}

function syno_encoder_show_verbose( $msg ) {
        global $verbose;
        if( $verbose )
                print "$msg\n";
}

?>
